/**
  ******************************************************************************
  * @file           : elabV2.c
  * @date           : 31.01.2023
  * @author:        : Stanislav Novak
  * @brief          : Main program body of Ard-eLabViewer V2;
  * 
  * Program also uses for its function PC application zero_elabviewer 
  * (https://github.com/adamberlinger/zero_elabviewer) by Adam Berlinger.
  * 
**/

//PIN_PWM=9;TIM1
//PIN_PWM=3;TIM2
//CNT_IN=8;
//AIN As (A1..A3)
//DIN PDs (0..7) 01RxTx;


//T1:
/*cnt as cnt and not sw
  watchdog; maybe for slow measuements
  scope single, more for trigger pre, ch2; ch3?
  scope protocol - disable non-la features for la
*/
#include <compat/deprecated.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/io.h>
#define BAUD 115200
#include <util/setbaud.h>


#define CHBAS 0
#define CHOSC 1
#define CHPWM 2
#define CHVM 3
#define CHCNT 4

#include "T1.h"

struct {
  uint8_t sBuffer[256];
  uint8_t first;
  uint8_t last;
} recBuffer = { {}, 0, 0 };

uint8_t c;
uint16_t i;

//VM
struct {  //TODO: improve
  uint64_t voltage[5];
  uint32_t svoltage[5];
  uint8_t* voltdat;
  uint32_t nextmeas;  //T2 in next version
  uint8_t vmRun;
  uint8_t vmSample[2];
} volt = { {}, {}, (uint8_t*)volt.svoltage + 2, 0, 0, {} };


//TIM2
struct {
  uint8_t pwmPre;
  uint8_t pwmPer;
  uint8_t pwmDut;  //0-100
} t2 = { 6, 255, 50 };


uint8_t conf = 0;  //0..3
const uint8_t caps[4] PROGMEM = { 13, 7, 21, 7 };
const uint8_t str[5][16] PROGMEM = { "ATmega328", "VM_PWM1", "OS_PWM1", "C__PWM2", "LA_PWM1" };  //four

//SCOPE
//uint8_t scope.scCh = 1;//std <<
//uint8_t triggerCh = 1;
//pretrigger,
struct {
  uint8_t scRun;
  uint8_t samples[768];  //main mem
  uint16_t sample;       //mem index
  uint32_t scLastSa;     //
  uint32_t scPeriod;
  uint32_t scFreq;
  uint8_t triggerState;   //1 dn, 2 up, 3 run! 0 undef
  uint8_t scTrigger;      //0 auto, 1 r, 2 f 3 rf
  uint8_t aTrigger;       //auto trigger
  uint16_t triggerLevel;  //triggerTODO
  uint8_t scCh;
  uint16_t schBufferLen;
} scope = { 0, {}, 0, 0, 1000, 1000, 3, 1, 0, 512, 1, 768 };


//counter
struct {  //TODO better names
  uint8_t counting;
  uint32_t cntPeriod;
  uint8_t* cntPeC;
  uint32_t cntDuty;  //const
  uint8_t* cntDuC;
  uint32_t cntFreqD;  //const
  uint8_t* cntFreq;
  uint16_t cntOvf;
  uint32_t cntLast;
} cnt = { 0, 0, (uint8_t*)(&(cnt.cntPeriod)), 1, (uint8_t*)(&(cnt.cntDuty)), 16e6, (uint8_t*)(&(cnt.cntFreqD)), 0, 0 };

uint32_t period(uint8_t pin, uint8_t state);
//uint32_t freq();

//functions
void setupCallback();
void voltLoop();
void scopeLoop();
void cntLoop();
void laLoop();

void sendBin(uint8_t ch, uint8_t len, uint8_t* data) {
  Serial.write(0xFF);
  Serial.write(ch);
  Serial.write((char)len & 0xff);
  Serial.write((char)len & 0xff00);
  for (c = 0; c < len; ++c) Serial.write(data[c]);
}

uint16_t getVcc() {
  ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  delay(1);             // Wait for Vref; bit to much?
  ADCSRA |= _BV(ADSC);  // Convert
  while (bit_is_set(ADCSRA, ADSC))
    ;
  return 1126400 / ADC;  //1.1*1024/ADC-theoretical
  //return 1101237/ADC;//calibrated value for current testing board
}



void sendT2Val() {
  Serial.write(0xFE);
  Serial.write(0x0);
  Serial.write(0x12);
  Serial.write(0x7A);
  Serial.write(0);
  Serial.write('F');
  Serial.write(recBuffer.sBuffer[(recBuffer.first + 6) & 0xff]);
  uint32_t t1val = OCR2A + 1;
  switch (t2.pwmPre) {
    case 7:
      t1val <<= 9;
      break;
    case 6:
      t1val <<= 7;
      break;
    case 5:
      t1val <<= 6;
      break;
    case 4:
      t1val <<= 5;
      break;
    case 3:
      t1val <<= 4;
      break;
    case 2:
      t1val <<= 2;
      break;
    case 1:
      t1val >>= 1;
    default:
      break;
  }
  Serial.write(0xFE);
  Serial.write((char)(t1val & 0xff));
  Serial.write((char)((t1val & 0xff00) >> 8));
  Serial.write((char)((t1val & 0xff0000) >> 16));
  Serial.write((char)((t1val & 0xff000000) >> 24));
  Serial.write('D');
  Serial.write(recBuffer.sBuffer[(recBuffer.first + 6) & 0xff]);
}

void changeTIM2(uint32_t freq) {
  TCCR2B = 8;  //pwm disabled
  TCNT2 = 0;

  if (freq & (0x1 << 14)) {
    freq = 1000 * (freq & 0x3FFF);
  } else if (freq & (0x2 << 14)) {
    freq = 1000000 * (freq & 0x3FFF);
  }
  //freq now cycles
  if (freq < 62) {
    t2.pwmPre = 7;
    t2.pwmPer = 255;
  } else if (freq < 245) {
    t2.pwmPre = 7;
    t2.pwmPer = 16000000 / (freq << 10);
  } else if (freq < 489) {
    t2.pwmPre = 6;
    t2.pwmPer = 16000000 / (freq << 8);
  } else if (freq < 977) {
    t2.pwmPre = 5;
    t2.pwmPer = 16000000 / (freq << 7);
  } else if (freq < 1954) {
    t2.pwmPre = 4;
    t2.pwmPer = 16000000 / (freq << 6);
  } else if (freq < 7813) {
    t2.pwmPre = 3;
    t2.pwmPer = 16000000 / (freq << 5);
  } else if (freq < 62500) {
    t2.pwmPre = 2;
    t2.pwmPer = 16000000 / (freq << 3);
  } else if (freq >= 62500) {
    t2.pwmPre = 1;
    if (freq > 100000) freq = 100000;  //max freq
    t2.pwmPer = 16000000 / freq;
  }
}


void processBuffer() {
  if (recBuffer.sBuffer[recBuffer.first] == 0xFE) {
    switch (recBuffer.sBuffer[(recBuffer.first + 6) & 0xff]) {
      case CHBAS:
        if (recBuffer.sBuffer[(recBuffer.first + 5) & 0xff] == 'C') recBuffer.first += 6;
        if (recBuffer.sBuffer[(recBuffer.first + 5) & 0xff] == 'G') {
          if (recBuffer.sBuffer[(recBuffer.first + 1) & 0xff] == 'N') {

            Serial.write(0xFF);
            Serial.write(CHBAS);
            Serial.write(10);
            Serial.write(0);
            for (c = 0; c < 9; ++c) Serial.write(pgm_read_byte_near((*str) + c));
            Serial.write(0);
            conf = (conf + 1) & 3;
            recBuffer.first += 6;
          }
          if (recBuffer.sBuffer[(recBuffer.first + 1) & 0xff] == 'V') {
            i = getVcc();
            Serial.write(0xFF);
            Serial.write(CHBAS);
            Serial.write(4);
            Serial.write(0);
            Serial.write(i & 0x00ff);
            Serial.write((i & 0xff00) >> 8);
            Serial.write(0);
            Serial.write(0);
            recBuffer.first += 6;
          }
          if (recBuffer.sBuffer[(recBuffer.first + 1) & 0xff] == 'C') {  //TODO

            Serial.write(0xFF);
            Serial.write(CHBAS);
            Serial.write(7);
            Serial.write(0);
            for (c = 0; c < 6; ++c) Serial.write(pgm_read_byte_near(str[conf + 1] + c));
            Serial.write(0);
            recBuffer.first += 6;
          }
          if (recBuffer.sBuffer[(recBuffer.first + 1) & 0xff] == 'F') {  //TODO
            Serial.write(0xFF);
            Serial.write(CHBAS);
            Serial.write(4);
            Serial.write(0);
            Serial.write(pgm_read_byte_near(caps + conf));
            Serial.write(0);
            Serial.write(0);
            Serial.write(0);
            setupCallback();
            recBuffer.first += 6;
          }
        } else recBuffer.first += 1;
        break;
      case CHPWM:
        if (conf != 2) {
          switch (recBuffer.sBuffer[(recBuffer.first + 5) & 0xff]) {
            case 'F':  //freq
              changefreq((uint32_t)(recBuffer.sBuffer[(recBuffer.first + 1) & 0xff] + (recBuffer.sBuffer[(recBuffer.first + 2) & 0xff] << 8) + (recBuffer.sBuffer[(recBuffer.first + 3) & 0xff] << 16) + (recBuffer.sBuffer[(recBuffer.first + 4) & 0xff] << 24)));
              recBuffer.first += 6;
              break;
            case 'D':  //duty-cyc
              TCCR1A &= ~_BV(COM1A1);
              pwmDC = (((long)((recBuffer.sBuffer[(recBuffer.first + 1) & 0xff]) | (recBuffer.sBuffer[(recBuffer.first + 2) & 0xff] & 0x3) << 8)) << 10) / 1000;
              setPWMDuty(pwmDC);
              if (pwmRun) TCCR1A |= _BV(COM1A1);
              recBuffer.first += 6;
              break;
            case 'S':  //start-stop
              pwmRun = recBuffer.sBuffer[(recBuffer.first + 1) & 0xff];
              if (pwmRun) TCCR1A |= _BV(COM1A1);
              else TCCR1A &= ~_BV(COM1A1);
              recBuffer.first += 6;
              break;
            default:
              recBuffer.first += 6;
              break;
          }
        } else {
          switch (recBuffer.sBuffer[(recBuffer.first + 5) & 0xff]) {
            case 'F':  //freq
              changeTIM2((uint32_t)(recBuffer.sBuffer[(recBuffer.first + 1) & 0xff] + (recBuffer.sBuffer[(recBuffer.first + 2) & 0xff] << 8) + (recBuffer.sBuffer[(recBuffer.first + 3) & 0xff] << 16) + (recBuffer.sBuffer[(recBuffer.first + 4) & 0xff] << 24)));

              t2.pwmPer -= 1;
              OCR2A = t2.pwmPer;
              OCR2B = t2.pwmPer * t2.pwmDut / 100;
              TCCR2B = pwmRun ? (8 | t2.pwmPre) : 8;
              sendT2Val();
              recBuffer.first += 6;
              break;
            case 'D':  //duty-cyc
              t2.pwmDut = ((((recBuffer.sBuffer[(recBuffer.first + 1) & 0xff])) | (recBuffer.sBuffer[(recBuffer.first + 2) & 0xff] & 0x3) << 8)) / 10;

              if (!t2.pwmDut) TCCR2A = (1 << WGM21) | (1 << WGM20);       // zero correction to not be a short pulse
              else TCCR2A = (1 << WGM21) | (1 << WGM20) | (1 << COM2B1);  // TCCR2A = pwmDut ? 0x23: 0x3;

              OCR2B = t2.pwmPer * t2.pwmDut / 100;
              TCCR2B = pwmRun ? (8 | t2.pwmPre) : 8;
              recBuffer.first += 6;
              break;
            case 'S':  //start-stop
              pwmRun = recBuffer.sBuffer[(recBuffer.first + 1) & 0xff];
              TCCR2B = pwmRun ? (8 | t2.pwmPre) : 8;
              recBuffer.first += 6;
              break;
            default:
              recBuffer.first += 1;
              break;
          }
        }
        break;
      case CHVM:
        if (conf == 0) {
          switch (recBuffer.sBuffer[(recBuffer.first + 5) & 0xff]) {
            case 'A':                                                              //average
              *(volt.vmSample) = recBuffer.sBuffer[(recBuffer.first + 1) & 0xff];  //only 8 bit
              recBuffer.first += 6;
              break;
            case 'S':  //start-stop
              volt.vmRun = recBuffer.sBuffer[(recBuffer.first + 1) & 0xff];
              if (volt.vmRun) {
                volt.nextmeas = millis();
              }
              recBuffer.first += 6;
              break;
            default:
              recBuffer.first += 1;
              break;
          }
        } else recBuffer.first += 1;
        break;
      case CHOSC:
        if (conf & 1) {  //2nd or 4th conf
          switch (recBuffer.sBuffer[(recBuffer.first + 5) & 0xff]) {
            case 'S':  //start-stop
              scope.scRun = recBuffer.sBuffer[(recBuffer.first + 1) & 0xff];
              if (scope.scRun) {
                TCCR2B = (8 | t2.pwmPre);
                sendT2Val();
                sendT2Val();
              } else TCCR2B = 8;
              recBuffer.first += 6;
              break;
            case 'F':  //freq Hz
              scope.scFreq = (uint32_t)((recBuffer.sBuffer[(recBuffer.first + 1) & 0xff] + (recBuffer.sBuffer[(recBuffer.first + 2) & 0xff] << 8) + (recBuffer.sBuffer[(recBuffer.first + 3) & 0xff] << 16) + (recBuffer.sBuffer[(recBuffer.first + 4) & 0xff] << 24)));
              if (scope.scFreq & (0x1 << 14)) {
                if (conf & 2) {  //LA
                  if (100 < (scope.scFreq & 0x3fff)) scope.scFreq = 100 | (0x1 << 14);
                } else {
                  if (80 < (scope.scFreq & 0x3fff)) scope.scFreq = 80 | (0x1 << 14);
                }
                if (!(conf & 2) && (scope.scCh == 3) && (50 < (scope.scFreq & 0x3fff))) scope.scFreq = 50 | (0x1 << 14);
                scope.scPeriod = 1000 / (scope.scFreq & 0x3fff);
              } else scope.scPeriod = 1000000 / scope.scFreq;
              changeTIM2(scope.scFreq);
              t2.pwmPer -= 1;
              OCR2A = t2.pwmPer;
              OCR2B = t2.pwmPer >> 2;
              TCCR2B = scope.scRun ? (8 | t2.pwmPre) : 8;
              sendT2Val();
              sendT2Val();
              recBuffer.first += 6;
              break;
            case 'T':  //trigger level mV
              scope.triggerLevel = ((recBuffer.sBuffer[(recBuffer.first + 1) & 0xff]) + ((recBuffer.sBuffer[(recBuffer.first + 2) & 0xff]) << 8)) * 0.267;
              recBuffer.first += 6;
              break;
            case 'R':  //trigger ch;TODO
              recBuffer.first += 6;
              break;
            case 'D':  //pretrigger ;TODO
              recBuffer.first += 6;
              break;
            case 'M':  //1->auto 0->r/f;TODO
              scope.aTrigger = recBuffer.sBuffer[(recBuffer.first + 1) & 0xff];
              recBuffer.first += 6;
              break;
            case 'P':  //1->R 2->F
              scope.scTrigger = recBuffer.sBuffer[(recBuffer.first + 1) & 0xff];
              recBuffer.first += 6;
              break;
            case 'C':
              scope.scCh = recBuffer.sBuffer[(recBuffer.first + 1) & 0xff];
              if ((scope.scFreq & (0x1 << 14)) && !(conf & 2) && (scope.scCh == 3) && (50 < (scope.scFreq & 0x3fff))) {
                scope.scFreq = 50 | (0x1 << 14);
                scope.scPeriod = 1000 / (scope.scFreq & 0x3fff);
                changeTIM2(scope.scFreq);
                t2.pwmPer -= 1;
                OCR2A = t2.pwmPer;
                OCR2B = t2.pwmPer >> 2;
                TCCR2B = scope.scRun ? (8 | t2.pwmPre) : 8;
                sendT2Val();
                sendT2Val();
              }
              recBuffer.first += 6;
              break;
            case 'B':
              if ((recBuffer.sBuffer[(recBuffer.first + 1) & 0xff]) + ((recBuffer.sBuffer[(recBuffer.first + 2) & 0xff]) << 8) > 767) scope.schBufferLen = 768;
              else scope.schBufferLen = recBuffer.sBuffer[(recBuffer.first + 1) & 0xff] + ((recBuffer.sBuffer[(recBuffer.first + 2) & 0xff]) << 8);
              recBuffer.first += 6;
              break;
            default:
              recBuffer.first += 1;
              break;
          }
        } else recBuffer.first += 1;
        break;
      case CHCNT:
        if (conf == 2) {
          if (recBuffer.sBuffer[(recBuffer.first + 5) & 0xff] == 'S') cnt.counting = recBuffer.sBuffer[(recBuffer.first + 1) & 0xff];
          scope.scLastSa = micros();
          recBuffer.first += 6;
        } else recBuffer.first += 6;
        break;
      default:
        recBuffer.first += 1;
        break;
    }
  } else recBuffer.first += 1;
}



void setup() {
  Serial.begin(115200);
  Serial.write(0xFF);
  Serial.write(CHBAS);
  Serial.write(8);
  Serial.write(0);  //send config to PC (typical is reset during setup)
  for (char i = 0; i < 7; ++i) Serial.write(pgm_read_byte_near(str[0] + i));
  Serial.write(0);
  ADCSRA = (ADCSRA & B11111000) | 3;
  pwmRun = 0;
}
uint8_t j;
void loop() {
  while (Serial.available()) {
    j = Serial.available();
    for (; j != 0; --j) {
      recBuffer.sBuffer[recBuffer.last] = Serial.read();
      ++recBuffer.last;
    }
  }
  while (((recBuffer.last - recBuffer.first + 0xff + 1) & 0xff) > 6) processBuffer();

  switch (conf) {  //FUNCTIONAL INIT
    case 0:        //VM loop
      voltLoop();
      break;
    case 1:  //OSC loop
      scopeLoop();
      break;
    case 2:  //CNT loop
      cntLoop();
      break;
    case 3:  //LA lool
      laLoop();
      break;
    default:
      break;
  }
}
void voltLoop() {
  if (volt.vmRun && (volt.nextmeas + 10 < millis())) {  //time for measure; in future rewrite with overflow tag of counter
    volt.nextmeas += 10;
    ADCSRA = (ADCSRA & B11111000) | 7;
    volt.voltage[1] += analogRead(A0);
    volt.voltage[2] += analogRead(A1);
    volt.voltage[3] += analogRead(A2);
    volt.voltage[4] += getVcc();
    ++(volt.vmSample[1]);  //sampling
    //sendBin(CHVM, 18 , voltdat);
  }

  if (volt.vmSample[0] <= volt.vmSample[1]) {
    volt.voltdat[0] = volt.vmSample[1];              //vmSample[1];//set to 1
    volt.svoltage[4] = (uint32_t)(volt.voltage[4]);  ///vmSample[1]
    volt.voltage[4] = 0;                             //vmSample[1];
    for (char i = 1; i < 4; ++i) {
      volt.svoltage[i] = (uint32_t)(volt.voltage[i] * volt.svoltage[4] * 0.001212121212 / volt.vmSample[1]);  //vcc correction*(4*5/3.3)/5000
      volt.voltage[i] = 0;
    }
    volt.vmSample[1] = 0;
    sendBin(CHVM, 18, volt.voltdat);
  }
}

void scopeLoop() {
  if (scope.scRun && (scope.sample == 0)) {

    if (scope.scPeriod < 50) {
      ADCSRA = (ADCSRA & B11111000) | 2;
    } else {
      if (scope.scPeriod < 100) {
        ADCSRA = (ADCSRA & B11111000) | 3;
      } else {
        if (scope.scPeriod < 200) {
          ADCSRA = (ADCSRA & B11111000) | 4;
        } else {
          if (scope.scPeriod < 400) {
            ADCSRA = (ADCSRA & B11111000) | 5;
          } else {
            if (scope.scPeriod < 800) {
              ADCSRA = (ADCSRA & B11111000) | 6;
            } else {
              ADCSRA = (ADCSRA & B11111000) | 7;
            }
          }
        }
      }
    }
    if (scope.aTrigger) {
      scope.triggerState = 3;
      TCNT2 = 0;
      TCCR2B = (8 | t2.pwmPre);
      handleT2Sc();
    } else {
      switch (scope.scTrigger) {
        case 0:
          break;
        case 1:  //R
          if (analogRead(A0) >= scope.triggerLevel) {
            if (scope.triggerState == 1) {
              scope.triggerState = 3;
              TCNT2 = 0;
              TCCR2B = (8 | t2.pwmPre);
              handleT2Sc();
            } else scope.triggerState = 2;
          } else scope.triggerState = 1;
          break;
        case 2:  //F
          if (analogRead(A0) <= scope.triggerLevel) {
            if (scope.triggerState == 2) {
              scope.triggerState = 3;
              TCNT2 = 0;
              TCCR2B = (8 | t2.pwmPre);
              handleT2Sc();
            } else scope.triggerState = 1;
          } else scope.triggerState = 2;
          break;
        case 3:  //RF
          if ((analogRead(A0) >= scope.triggerLevel)) {
            if (scope.triggerState == 1){//appended
              scope.triggerState = 3;
              TCNT2 = 0;
              TCCR2B = (8 | t2.pwmPre);
              handleT2Sc();
            }
            else scope.triggerState = 2;
          } else {
            if (scope.triggerState == 2){//appended
              scope.triggerState = 3;
              TCNT2 = 0;
              TCCR2B = (8 | t2.pwmPre);
              handleT2Sc();
            }
            else scope.triggerState = 1;
          }
          break;
        default:
          break;
      }
    }
  }

  if (scope.scRun && (scope.sample >= scope.schBufferLen)) {  //end of buffer => send//sendbin//TODO shorter buffer opt
    Serial.write(0xFF);
    Serial.write(CHOSC);
    Serial.write((char)(((scope.schBufferLen + 1) << 1) & 0xff));
    Serial.write((char)(((scope.schBufferLen + 1) & 0x7f80) >> 7));
    Serial.write((char)1);
    if (scope.scCh == 3) Serial.write((char)40);
    else Serial.write((char)24);
    for (i = 0; i < scope.schBufferLen; ++i) {
      Serial.write((char)((scope.samples[i]) << 4));
      Serial.write((char)((scope.samples[i]) >> 4));
    }
    scope.sample = 0;
    scope.triggerState = 0;
    if (scope.scRun == 2) scope.scRun = 0;
  }
};
void setupCallback() {
  pwmRun = 0;
  volt.vmRun = 0;
  scope.scRun = 0;
  cnt.counting = 0;
  if (conf != 2) {  //INIT PWM1
    TCCR2B = 8;     //pwm2 disabled

    TIMSK1 = 0;
    TIFR1 = 0;

    TCCR1A = 0;
    TCCR1B = _BV(WGM13);  //Phase correct pwm
    t1Period(1000UL);
    pinMode(9, OUTPUT);
    pwmRun = 0;
    TCCR1A &= ~_BV(COM1A1);  //stopPwm
    pwmDC = 512;
    setPWMDuty(512);  //set 50% duty cyc
    changefreq(100);
    sendTimerVal();

  } else {                   //INIT PWM2
    TCCR1A &= ~_BV(COM1A1);  //stopPwm1

    //DDRD |= (1 << PD3);// set 3 as output
    pinMode(3, OUTPUT);
    TCCR2A = (1 << WGM21) | (1 << WGM20) | (1 << COM2B1);  // Fast PWM
    TCCR2B = 8;                                            //pwm disabled

    TCNT2 = 0;
    OCR2A = 255;
    OCR2B = 50;  //pwm duty cycle at 50 %
    sendT2Val();

    //init counter
    TCCR1B = 0;
    TCNT1 = 0;
    TIFR1 = (1 << ICF1) | (1 << TOV1);
    TIMSK1 = (1 << ICIE1) | (1 << TOIE1);
    cnt.cntLast = 0;
    cnt.cntOvf = 0;
    cnt.cntPeriod = 0;
    TCCR1B = (1 << ICNC1) | (1 << ICES1) | (1 << CS10);
  }  //sending timer2 val

  if (conf & 1) {                                     //T2 for scope timing
    TCCR2A = (1 << WGM21) | (1 << WGM20) | (1 << COM2B1);  // Fast PWM
    TCCR2B = (8 | t2.pwmPre);
    t2.pwmPre = 4;  //basic 1khz
    TCNT2 = 0;
    OCR2A = 249;
    OCR2B = 124;  //pwm duty cycle at 50 %
    TIMSK2 |= (1 << OCIE2A);
  } else TIMSK2 = 0;

  if (conf != 1) ADMUX &= ~(1 << 5);
  switch (conf) {  //FUNCTIONAL INIT
    case 0:
      *(volt.vmSample) = 1;
      *(volt.vmSample + 1) = 0;
      break;
    case 3:
      pinMode(3, INPUT);
    default:
      break;
  }
}

void laLoop() {
  if (scope.scRun && (scope.sample == 0)) {
    if (scope.aTrigger){
              scope.triggerState = 3;
              TCNT2 = 0;
              TCCR2B = (8 | t2.pwmPre);
              handleT2LA();
            }
    else {
      switch (scope.scTrigger) {
        case 0:
          break;
        case 1:                   //R
          if (PIND & (1 << 2)) {  //pin2 low
            if (scope.triggerState == 1) {
              scope.triggerState = 3;
              TCNT2 = 0;
              TCCR2B = (8 | t2.pwmPre);
              handleT2LA();
            }
            else scope.triggerState = 2;
          } else scope.triggerState = 1;
          break;
        case 2:  //F
          if (!(PIND & (1 << 2))) {
            if (scope.triggerState == 2) {
              scope.triggerState = 3;
              TCNT2 = 0;
              TCCR2B = (8 | t2.pwmPre);
              handleT2LA();
            }
            else scope.triggerState = 1;
          } else scope.triggerState = 2;
          break;
        case 3:  //RF
          if (PIND & (1 << 2)) {
            if (scope.triggerState == 1) {
              scope.triggerState = 3;
              TCNT2 = 0;
              TCCR2B = (8 | t2.pwmPre);
              handleT2LA();
            }
            else scope.triggerState = 2;
          } else {
            if (scope.triggerState == 2) {
              scope.triggerState = 3;
              TCNT2 = 0;
              TCCR2B = (8 | t2.pwmPre);
              handleT2LA();
            }
            else scope.triggerState = 1;
          }
          break;
        default:
          break;
      }
    }
  }
  //removed cyclic read
  if (scope.scRun && (scope.sample == 768)) {  //end of buffer => send//sendbin
    Serial.write(0xFF);
    Serial.write(CHOSC);
    Serial.write((char)(3) & 0xff);
    Serial.write((char)6);
    Serial.write((char)2);
    Serial.write((char)20);
    Serial.write((char)21);  //edited
    for (i = 0; i < 768; ++i) Serial.write('\0');
    for (i = 0; i < 768; ++i) {
      Serial.write(scope.samples[i] & 0xfc);
      //Serial.write((char)((scope.samples[i])<< 4));
      //Serial.write((char)((scope.samples[i])>> 4));
    }
    scope.sample = 0;
    scope.triggerState = 0;
    if (scope.scRun == 2) scope.scRun = 0;
  }
}
void cntLoop() {
  if (cnt.counting && (scope.scLastSa < micros())) {
    Serial.write(0xFF);
    Serial.write(CHCNT);
    Serial.write(12);
    Serial.write(0);
    Serial.write(cnt.cntFreq[0]);
    Serial.write(cnt.cntFreq[1]);
    Serial.write(cnt.cntFreq[2]);
    Serial.write(cnt.cntFreq[3]);
    Serial.write(cnt.cntPeC[0]);
    Serial.write(cnt.cntPeC[1]);
    Serial.write(cnt.cntPeC[2]);
    Serial.write(cnt.cntPeC[3]);
    Serial.write(cnt.cntDuC[0]);
    Serial.write(cnt.cntDuC[1]);
    Serial.write(cnt.cntDuC[2]);
    Serial.write(cnt.cntDuC[3]);
    scope.scLastSa = 400000 + micros();
  }
}

ISR(TIMER1_OVF_vect) {
  cli();
  ++cnt.cntOvf;
  sei();
}

ISR(TIMER1_CAPT_vect) {
  cli();
  if (TIFR1 & (1 << TOV1) && ICR1 < 0xFF00) {
    TIFR1 = (1 << TOV1);
    ++cnt.cntOvf;
  }
  cnt.cntPeriod = (((uint32_t)cnt.cntOvf << 16) | ICR1) - cnt.cntLast;
  cnt.cntLast += cnt.cntPeriod;
  sei();
}
void handleT2Sc() {                                //TODO LA variant
  if (scope.scRun && (scope.triggerState == 3)) {  //10kHz, single
    if (scope.scCh & 1) {
      {
        ADMUX = (1 << 5) | (1 << 6) | (0 & 0x07);
        sbi(ADCSRA, ADSC);
        c = 1 << ((ADCSRA & 0x7) + 1);
        while (c) --c;
        if (scope.scCh & 2) ADMUX = (1 << 5) | (1 << 6) | (1 & 0x07);
        while (bit_is_set(ADCSRA, ADSC))
          ;
        scope.samples[scope.sample] = ADCH;
      }
      //scope.samples[scope.sample] = (char)((analogRead(A0)) >> 2);
      ++scope.sample;
    }
    if (scope.scCh & 2) {
      {
        ADMUX = (1 << 5) | (1 << 6) | (1 & 0x07);
        sbi(ADCSRA, ADSC);
        c = 1 << ((ADCSRA & 0x7) + 1);
        while (c) --c;
        if (scope.scCh & 1) ADMUX = (1 << 5) | (1 << 6) | (0 & 0x07);
        while (bit_is_set(ADCSRA, ADSC))
          ;
        scope.samples[scope.sample] = ADCH;
      }
      //scope.samples[scope.sample] = (char)((analogRead(A0)) >> 2);
      ++scope.sample;
    }
    if (scope.sample >= scope.schBufferLen) {  //end of buffer => send//sendbin//TODO shorter buffer opt
      TCCR2B = 8;
    }
  }
}

void handleT2LA() {
  if (scope.scRun && (scope.triggerState == 3)) {
    scope.samples[scope.sample] = PIND;
    ++scope.sample;
    if (scope.sample >= scope.schBufferLen) {  //end of buffer
      TCCR2B = 8;
    }
  }
}

ISR(TIMER2_COMPA_vect) {
  if (conf == 1) handleT2Sc();
  else handleT2LA();
}
